# Agenten Plattform
#
# (c) 2024 Magnus Bender
# 	Institute of Humanities-Centered Artificial Intelligence (CHAI)
# 	Universitaet Hamburg
# 	https://www.chai.uni-hamburg.de/~bender
#  
# source code released under the terms of GNU Public License Version 3
# https://www.gnu.org/licenses/gpl-3.0.txt

import re
from typing import Callable

from ums.agent import GatekeeperAgent
from ums.utils.types import Riddle, RiddleSolution, AgentMessage


class SimpleSolveAgent(GatekeeperAgent):

	def before_response(self, response: AgentMessage, send_it: Callable[[], None]) -> bool:
		# do not send a response, if this is not a calculator riddle!
		return not self.stop_response

	def handle(self, solution: RiddleSolution, riddle: Riddle) -> RiddleSolution:
		self.stop_response = False

		# first check for errors
		if solution.solution == "Error":
			solution.accepted = True 
			solution.review = "An error of the riddle can not be fixed!"
			return solution

		# this is just a simple check, we check if solution and explanation match to the expression 

		match = re.match(r"^([0-9+\-*\/ ]+)\s*=\s*(\d+)$", solution.explanation)
		if match is None:
			self.stop_response = True
			return solution
		
		expression, result = match.group(1), match.group(2)

		if result != solution.solution:
			solution.accepted = False 
			solution.review = "Inconsistent values"
			return solution

		try:
			# using eval is a bad idea, but this is only for demonstration
			#	and expression may only contain "0-9 +-*/"
			own_result = eval(expression)
		except:
			solution.accepted = False 
			solution.review = "Unsolvable expression"
			return solution
		
		# check the values
		if str(own_result) != solution.solution:
			solution.accepted = False 
			solution.review = "Value of expression does not match solution!"
		else:
			solution.accepted = True 
			solution.review = "Yes, {}".format(solution.explanation)
		
		return solution
	
AGENT_CLASSES = [
	SimpleSolveAgent
]